#!/bin/bash 

####################################################################################
### Цвета для подсказок
####################################################################################

#Black        0;30     Dark Gray     1;30
#Red          0;31     Light Red     1;31
#Green        0;32     Light Green   1;32
#Brown/Orange 0;33     Yellow        1;33
#Blue         0;34     Light Blue    1;34
#Purple       0;35     Light Purple  1;35
#Cyan         0;36     Light Cyan    1;36
#Light Gray   0;37     White         1;37

RED='\033[0;31m'
LIGHTRED='\033[1;35m'
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
LIGHTGREEN='\033[1;32m'
DARKGRAY='\033[1;30m'
NC='\033[0m' # No Color

BOLD='\033[1m'
NORMAL='\033[0m'

####################################################################################
### Завершаем работу скрипта при ошибках
####################################################################################

set -e

####################################################################################
### Задаём переменные
####################################################################################

# Переменные, настраиваемые пользователем, храним отдельно. 
source <(grep = /app/asr_billing/cfg/sorm3-mfi/config.ini)

####################################################################################
### Основные функции
####################################################################################

help(){
    # Проверяем, что скрипту сказали что настроить
    # Если нет - даём подсказку и выходим
    echo 
    echo "Укажите, что нужно настроить:"
    echo
    echo -e "   ${LIGHTGREEN}prepare_references${NC} - создать реквизит с типами документов, удостоверяющих личность"
    echo -e "   ${LIGHTGREEN}prepare_abonents${NC} - добавить всем абонентам реквизит с типом документа Паспорт гражданина РФ"
    echo -e "   ${LIGHTGREEN}make_doc_type${NC} - сформировать справочник с типами документов для выгрузки на СОРМ"
    echo -e "   ${LIGHTGREEN}install_cron${NC} - настроить CRON"
    echo -e "   ${LIGHTGREEN}install_logrotate${NC} - настроить logrotate"
    echo -e "   ${LIGHTGREEN}prepare_payment_report${NC} - выгрузить справочник используемых платёжных систем и подготовить sql для выгрузки платежей"
    echo -e "   ${LIGHTGREEN}all${NC} - всё вышеперечисленное"
    echo -e "${NC}"
    echo 
    echo "Например:"
    echo 
    echo -e "  ${RED}configure.sh ${GREEN}all${NC}"
    echo 
    exit 0
}

# Функция добавляет служебный реквизит с типами документов 
prepare_references(){
    # Проверяем есть ли реквизит
    does_attr_exist=$(sqlexec "set heading off; select count(*) from user_attributes where attribute_id=-332891" | sed 's/ *//g; /^$/d')

    if [[ $does_attr_exist == 0 ]]; then 
        # Если нет - добавляем
        echo "Нет реквизита \"Тип документа\" с ID -332891, добавляем"
        curl -XPOST 'http://169.254.80.82:8082/rest_api/v2/UserAttributes/' -d 'method1=objects.create&arg1={"attribute_id":-332891,"name": "Тип документа","description": "Документ удостоверяющий личность","type_id": 3,"type_attr": "Неизвестно,Военный билет,Временное удостоверение личности гражданина РФ,Иностранный паспорт,Паспорт гражданина РФ,Паспорт гражданина СССР,Свидетельство о регистрации ходатайства иммигранта,Свидетельство о рождении,Справка об освобождении из мест лишения свободы,Удостоверение беженца в РФ,Удостоверение личности офицера,Вид на жительство в РФ иностранных граждан,Загранпаспорт гражданина СССР,Загранпаспорт гражданина РФ,Паспорт Минморфлота,Дипломатический паспорт гражданина РФ,Удостоверение личности,Свидетельство о рождении выданное уполномоченным органом иностранного государства,Паспорт моряка,Иные документы выданные МВД","framework_form_groups_id": null}'
    else 
        # Если есть - пишем об этом и идём дальше
        echo "Реквизит \"Тип документа\" с ID -332891 уже создан"
    fi 
}

# Функция добавляет реквизит с типом документов физлица у которых его ещё нет
prepare_abonents(){
    declare -a abonents 

    # Получаем список абонентов без реквизита "Тип документа"
    abonents=( $(sqlexec -l "
    select distinct 
        a.id 
    from 
        abonents a 
    left join 
        attribute_values av 
        on a.id=av.abonent_id 
        and attribute_id=-332891 
    where 
        coalesce(a.company,0)=0 
        and coalesce(a.category_id,1)=1 
        and a.is_folder=0 
        and a.id>0 and (av.id is null)" | awk '$2{print $2}' | tr '\n' ' ' ) )

    # Пишем количество абонентов без реквизита для лога
    echo  "Абонентов без типа документа: ${#abonents[*]}"

    for abon_id in ${abonents[*]}; do 
        # Пишем ID абонента для лога
        echo "Добавляем абоненту $abon_id реквизит \"Тип документа\" со значением по-умолчанию \"Паспорт гражданина РФ\""
        curl -XPOST 'http://169.254.80.82:8082/rest_api/v2/AttributeValues/' --data 'method1=objects.create&arg1={"abonent_id":'$abon_id',"attribute_id":"-332891","attribute_value":"Паспорт гражданина РФ"}'
    done
}

# Генерирует справочник типов документов (удостоверяющих личность) и оставляет его в data_dir
make_doc_type(){
    doc_type_path=$(/usr/bin/python2.6 ${scrits_dir}/make-doc_type_txt.py)
    echo "Сформирован справочник DOC_TYPE, выгружаем"
    # Выгружаем актуальный справочник типов платежей
    $scrits_dir/upload.sh pay_type
}

# Функция добавляет в планировщик ОС задание выгрузки в СОРМ 
install_cron(){
    target="/app/asr_billing/cfg/sorm3-mfi/etc-host/cron.d/sorm3-mfi-upload"
    symlink="/etc/cron.d/sorm3-mfi-upload"
    if [[ ! -L $symlink ]]; then
        echo "Настраиваем cron"
        ln -s $target $symlink
        chmod 644 $target
        service crond restart &
    elif [[ ! $(stat -c "%a" $target) == "644" ]]; then
        echo "Правим права на скрипт cron"
        chmod 644 $target
        service crond restart &        
    else 
        echo "cron уже настроен: файл или символьная ссылка /etc/cron.d/sorm3-mfi-upload существуют, права на скрипт правильные"
    fi 
}

# Функция настраивает ротировние лога выгрузки в СОРМ 
install_logrotate(){
    if [[ ! -L /app/asr_billing/etc/logrotate.d/sorm3 ]]; then
        echo "Настраиваем logrotate"
        ln -s /app/asr_billing/cfg/sorm3-mfi/etc-chroot/logrotate.d/sorm3 /app/asr_billing/etc/logrotate.d/sorm3
        chroot /app/asr_billing/ service rsyslog restart &
    else
        echo "logrotate уже настроен: файл или символьная ссылка /app/asr_billing/etc/logrotate.d/sorm3 существуют"
    fi 
}

prepare_payment_report(){

    ##
    ## В этой функции много "прибито гвоздями", но подойдёт для большинства операторов
    ## Если что-то менять - то нужно делать очень аккуратно, сверившись с документацией и подержкой
    ##

    echo "Делаем undefined_payment.sql и pay_type.csv"

    # Определяем пути к файлам
    u_payment_template="$sql_dir/undefined_payment_template.sql"
    u_payment_tmp="$sql_dir/undefined_payment_temp.sql"
    u_payment="$sql_dir/undefined_payment.sql"
    p_types_csv="$data_dir/pay_type.csv_raw"

    # Удаляем все файлы что будем переделывать
    rm -f $u_payment_tmp $u_payment $p_types_csv 
    
    # Получаем список операторов связи
    isp_list=( $(sqlexec -l "set heading off; select id from abonents where is_folder=0 and deleted=0 and category_id=2" | awk '$2{print $2}' | tr '\n' ' ') )

    # Получаем список платёжных систем 
    pay_systems_integrated=( $(grep -shir "paywork.*new" /app/asr_fiscal/usr/local/www/htdocs/*  | sed 's/   *//g; s/this->//g ;s/k=n/k = n/g; s/,/, /g' | awk '{print $5}' | sed -e "/^$/d; s/);//g; s/'//g" | sort | uniq) )
    
    # Получаем список кастомных платёжек, банков https://docs.carbonsoft.ru/50660520
    # _REPLACE_ME_ на случай если в названии платёжки есть пробел, дальше вернём как было
    # Небольшой костыль чтобы подружить формирование одномерных массивов в Bash и названия платёжек с пробелами 
    pay_systems_custom=( $(grep pay_operator /app/asr_billing/cfg/autocsv/*.autocsv | cut -d"'" -f 2 | sed 's/ /_REPLACE_ME_/g' | tr '\n' ' ') )

    # Подготавлваем счётчик - чтобы ID платёжек сходились в справочнике типов платежей и реестре платежей
    p_system_num=1
    # ДЛЯ ВСЕХ СТАНДАРТНЫХ ПЛАТЁЖЕК
    # Заполняем временный файл для создания sql из шаблона и финальный csv pay_types
    for pay_system in ${pay_systems_integrated[*]}; do 
        # Отправляем строку во временный файл для формирования case в sql
        echo -e "\t\twhen fo.operator_name = '$pay_system' then '$p_system_num'" >> $u_payment_tmp
        # Отправляем строку в csv типов платежей для последющей выгрузки, по каждому оператору связи
        for isp in ${isp_list[*]}; do
            echo -e "$p_system_num;31.12.1970 23:59:59;31.12.2024 23:59:59;$pay_system;$isp" >> $p_types_csv
        done 
        ((p_system_num++))
    done
    # ПОВТОРЯЕМ ДЛЯ КАСТОМНЫХ ПЛАТЁЖЕК
    for pay_system in ${pay_systems_custom[*]}; do 
        # Избавляемся от костыля, который сделали выше, формируя массив
        pay_system=$( echo $pay_system | sed 's/_REPLACE_ME_/ /g' )
        # Отправляем строку во временный файл для формирования case в sql
        echo -e "\t\twhen fo.operator_name = '$pay_system' then '$p_system_num'" >> $u_payment_tmp
        # Отправляем строку в csv типов платежей для последющей выгрузки, по каждому оператору связи
        for isp in ${isp_list[*]}; do
            echo -e "$p_system_num;31.12.1970 23:59:59;31.12.2024 23:59:59;$pay_system;$isp" >> $p_types_csv
        done 
        ((p_system_num++))
    done
    # Добавляем для case последнюю строку с "неопределённыи типом"
    echo -e "\t\twhen fo.operator_name is null then '$p_system_num'\n\t\telse '$p_system_num'" >> $u_payment_tmp
    # Отправляем строку в csv типов платежей "неопределённый тип", по каждому оператору связи
    for isp in ${isp_list[*]}; do
        echo -e "$p_system_num;31.12.1970 23:59:59;31.12.2024 23:59:59;Поступление наличных;$isp" >> $p_types_csv
    done 

    # Собираем sql файл выгрузок платежей по частям
    head -n 3 $u_payment_template >> $u_payment
    cat $u_payment_tmp >> $u_payment
    tail $u_payment_template -n+5 >> $u_payment
    # Удаляем временный файл 
    rm -f $u_payment_tmp

    # Выгружаем актуальный справочник типов платежей
    $scrits_dir/upload.sh pay_type
}

####################################################################################
### Выполнение скрипта
####################################################################################

# Первым агрументом можно выполнить конкретное действие
# Если запустили без агругментов (=not_set) - делаем всё
case ${1:-not_set} in 
    not_set ) 
        help;;
    all )
        prepare_references
        prepare_abonents
        make_doc_type
        install_cron
        install_logrotate
        prepare_payment_report;;
    *       ) 
        $1;;
esac
