set heading off; 

/*
ADDRESS_TYPE_ID
0 - Адрес регистрации (обязателен для юридических и физических лиц)
1 - Почтовый адрес (дополнительный адрес для юридических лиц)
2 - Адрес доставки счета (дополнительный адрес для юридических лиц)
3 - Адрес установки устройства (телефонов) (дополнительный адрес для абонентов телефонов общего пользования)
4 - Резерв

ADDRESS_TYPE
0 - структурированный 
1 - неструктурированный
*/ 

--
-- АДРЕС ИЗ КАРТОЧКИ АБОНЕНТА
--
-- ABONENT.HOME_ID
--
select 
    a.id || ';' || -- ABONENT_ID
    a.operator_id || ';' || -- REGION_ID 
    '0' || ';' || -- ADDRESS_TYPE_ID
    '0' || ';' || -- ADDRESS_TYPE
    coalesce(addr.zip_code,'') || ';' || -- ZIP
    coalesce(addr.country,'') || ';' || -- COUNTRY
    coalesce(addr.region,'') || ';' || -- REGION
    '' || ';' || -- ZONE
    iif( coalesce(addr.settlement,'')<>'' 
        ,coalesce(addr.settlement,'')  
        ,coalesce(addr.city,'')
        ) || ';' || -- CITY
    coalesce(addr.street,'') || ';' || -- STREET
    coalesce(addr.s_number,'') || ';' || -- BUILDING
    coalesce(addr.s_liter,'') || ';' || -- BUILD_SECT
    coalesce(a.a_home_number,'') || ';' || -- APARTMENT
    '' || ';' || -- UNSTRUCT_INFO
    iif(addr.connect_date is not null
        ,cast(cast(addr.connect_date as date) as varchar(32)) || ' 00:00:00'
        ,'1970-12-31 23:59:59'
        ) || ';' || -- BEGIN_TIME
    '2049-12-31 23:59:59' || ';' || -- END_TIME
    a.id || ';' || -- INTERNAL_ID1
    a.id -- INTERNAL_ID2
from
    abonents a 
join 
    (
    select 
        u.abonent_id,
        h.* 
    from 
        users u 
    join 
        homes h
        on u.home_id=h.id 
    where 
        u.ip is not null or u.ipv6<>'' or u.phone is not null 
    ) addr on a.id=addr.abonent_id
    
where 
    a.id>0
    and a.is_folder=0

union 

--
-- АДРЕС ИЗ УЧЁТНОЙ ЗАПИСИ, СОГЛАСНО ДОКУМЕНТАЦИИ МФИ - ТОЛЬКО ДЛЯ ТЕЛЕФОНОВ
--
-- USERS.HOME_ID
--
select 
    a.id || ';' || -- ABONENT_ID
    a.operator_id || ';' || -- REGION_ID
    case 
        when addr.phone is not null then '3'
        --when u.ip is not null or u.ipv6<>'' then '4' -- <-на будущее
    end || ';' || -- ADDRESS_TYPE_ID
    '0' || ';' || -- ADDRESS_TYPE
    coalesce(addr.zip_code,'') || ';' || -- ZIP
    coalesce(addr.country,'') || ';' || -- COUNTRY
    coalesce(addr.region,'') || ';' || -- REGION
    '' || ';' || -- ZONE
    iif( coalesce(addr.settlement,'')<>'' 
        ,coalesce(addr.settlement,'')  
        ,coalesce(addr.city,'')
        ) || ';' || -- CITY
    coalesce(addr.street,'') || ';' || -- STREET
    coalesce(addr.s_number,'') || ';' || -- BUILDING
    coalesce(addr.s_liter,'') || ';' || -- BUILD_SECT
    '' || ';' || -- APARTMENT
    '' || ';' || -- UNSTRUCT_INFO
    iif(addr.connect_date is not null
        ,cast(cast(addr.connect_date as date) as varchar(32)) || ' 00:00:00'
        ,'1970-12-31 23:59:59'
        ) || ';' || -- BEGIN_TIME
    '2049-12-31 23:59:59' || ';' || -- END_TIME
    a.id || ';' || -- INTERNAL_ID1
    a.id -- INTERNAL_ID2
from
    abonents a 
join 
    (
    select 
        u.abonent_id,
        u.phone,
        u.ip,
        u.ipv6,
        h.* 
    from 
        users u 
    join 
        homes h
        on u.home_id=h.id 
    where 
        u.phone is not null 
        --or u.ip is not null or u.ipv6<>'' -- <-на будущее, если для интернет тоже потребуется
    ) addr on a.id=addr.abonent_id
    
where 
    a.id>0
    and a.is_folder=0

union

--
-- АДРЕС ИЗ РЕКВИЗИТОВ, МОДЕЛЬ ДОМА
-- Например, стандартный атрибут "Адрес" (ID=3) или "Адрес регистрация" (ID=15)
--
-- ATTRIBUTE_VALUES.ATTRIBUTE_VALUE 
-- USER_ATTRIBUTES.TYPE_ID=1
--
select 
    a.id || ';' || -- ABONENT_ID
    a.operator_id || ';' || -- REGION_ID
    '' || ';' || -- ADDRESS_TYPE_ID
    '0' || ';' || -- ADDRESS_TYPE
    coalesce(addr.zip_code,'') || ';' || -- ZIP
    coalesce(addr.country,'') || ';' || -- COUNTRY
    coalesce(addr.region,'') || ';' || -- REGION
    '' || ';' || -- ZONE
    iif( coalesce(addr.settlement,'')<>'' 
        ,coalesce(addr.settlement,'')  
        ,coalesce(addr.city,'')
        ) || ';' || -- CITY
    coalesce(addr.street,'') || ';' || -- STREET
    coalesce(addr.s_number,'') || ';' || -- BUILDING
    coalesce(addr.s_liter,'') || ';' || -- BUILD_SECT
    '' || ';' || -- APARTMENT
    '' || ';' || -- UNSTRUCT_INFO
    iif(addr.connect_date is not null
        ,cast(cast(addr.connect_date as date) as varchar(32)) || ' 00:00:00'
        ,'1970-12-31 23:59:59'
        ) || ';' || -- BEGIN_TIME
    '2049-12-31 23:59:59' || ';' || -- END_TIME
    a.id || ';' || -- INTERNAL_ID1
    a.id -- INTERNAL_ID2
from
    abonents a 
join 
    (
    select 
        av.abonent_id,
        av.attribute_value,
        h.*
    from 
        attribute_values av
    join 
        user_attributes ua 
        on av.attribute_id=ua.attribute_id
            and ua.type_id=4
            and lower(ua.type_attr)='homes'
            and coalesce(av.attribute_value,'')<>''
    join 
        homes h
        on av.attribute_value=h.id 
    where 
        lower(ua.name) containing 'адрес'
    ) addr on a.id=addr.abonent_id
    
where 
    a.id>0
    and a.is_folder=0

union 

--
-- АДРЕС ИЗ РЕКВИЗИТОВ, ТЕКСТОМ
-- Например, стандартный атрибут "Юридический адрес" (ID=25) или "Почтовый адрес" (ID=27)
--
-- ATTRIBUTE_VALUES.ATTRIBUTE_VALUE 
-- USER_ATTRIBUTES.TYPE_ID=1
--
select 
    a.id || ';' || -- ABONENT_ID
    a.operator_id || ';' || -- REGION_ID    
    case 
        when lower(addr.name) containing 'почтовый' then '1'
        when lower(addr.name) containing 'физический' then '2'
        when lower(addr.name) containing 'юридический' then '2'
        --when u.ip is not null or u.ipv6<>'' then '4' -- <-на будущее
    end || ';' || -- ADDRESS_TYPE_ID
    '1' || ';' || -- ADDRESS_TYPE
    '' || ';' || -- ZIP
    '' || ';' || -- COUNTRY
    '' || ';' || -- REGION
    '' || ';' || -- ZONE
    '' || ';' || -- CITY
    '' || ';' || -- STREET
    '' || ';' || -- BUILDING
    '' || ';' || -- BUILD_SECT
    '' || ';' || -- APARTMENT
    coalesce(addr.attribute_value,'') || ';' || -- UNSTRUCT_INFO
    '' || ';' || -- BEGIN_TIME
    '' || ';' || -- END_TIME
    a.id || ';' || -- INTERNAL_ID1
    a.id -- INTERNAL_ID2
from
    abonents a 
join 
    (
    select 
        av.abonent_id,
        av.attribute_value,
        av.attribute_id,
        ua.name
    from 
        attribute_values av
    join 
        user_attributes ua 
        on av.attribute_id=ua.attribute_id
    where 
        lower(ua.name) containing 'адрес'
        and ua.type_id=1
        and (
            lower(ua.name) containing 'почтовый'
            or lower(ua.name) containing 'физический'
            or lower(ua.name) containing 'юридический'
        )
    ) addr on a.id=addr.abonent_id
    
where 
    a.id>0
    and a.is_folder=0
/* 
*/
;
